@@ -21,7 +21,7 @@
  *
  *  You should have received a copy of the GNU Lesser General Public
  *  License along with this library; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
  *  Copyright (C) 2008-2010 SlimLogic Ltd
  *  Copyright (C) 2010 Wolfson Microelectronics PLC
@@ -230,6 +230,7 @@ int snd_use_case_get_list(snd_use_case_mgr_t *uc_mgr,
  * Known identifiers:
  *   - NULL 		- return current card
  *   - _verb		- return current verb
+ *   - _file		- return configuration file loaded for current card
  *
  *   - [=]{NAME}[/[{modifier}|{/device}][/{verb}]]
  *                      - value identifier {NAME}
@@ -280,8 +281,10 @@ int snd_use_case_get_list(snd_use_case_mgr_t *uc_mgr,
  *      - playback control device name
  *   - PlaybackVolume
  *      - playback control volume ID string
+ *	- can be parsed using snd_use_case_parse_ctl_elem_id()
  *   - PlaybackSwitch
  *      - playback control switch ID string
+ *	- can be parsed using snd_use_case_parse_ctl_elem_id()
  *   - CaptureRate
  *      - capture device sample rate
  *   - CaptureChannels
@@ -290,16 +293,22 @@ int snd_use_case_get_list(snd_use_case_mgr_t *uc_mgr,
  *      - capture control device name
  *   - CaptureVolume
  *      - capture control volume ID string
+ *	- can be parsed using snd_use_case_parse_ctl_elem_id()
  *   - CaptureSwitch
  *      - capture control switch ID string
+ *	- can be parsed using snd_use_case_parse_ctl_elem_id()
  *   - PlaybackMixer
  *      - name of playback mixer
  *   - PlaybackMixerID
  *      - mixer playback ID
+ *	- can be parsed using snd_use_case_parse_selem_id()
  *   - CaptureMixer
  *      - name of capture mixer
  *   - CaptureMixerID
+ *	- can be parsed using snd_use_case_parse_selem_id()
  *      - mixer capture ID
+ *   - EDIDFile
+ *      - Path to EDID file for HDMI devices
  *   - JackControl, JackDev, JackHWMute
  *      - Jack information for a device. The jack status can be reported via
  *        a kcontrol and/or via an input device. **JackControl** is the
@@ -319,6 +328,10 @@ int snd_use_case_get_list(snd_use_case_mgr_t *uc_mgr,
  *        trick upper software layers to e.g. automatically mute speakers when
  *        headphones are plugged in, but that's application policy
  *        configuration that doesn't belong to UCM configuration files.
+ *   - MinBufferLevel
+ *	- This is used on platform where reported buffer level is not accurate.
+ *	  E.g. "512", which holds 512 samples in device buffer. Note: this will
+ *	  increase latency.
  */
 int snd_use_case_get(snd_use_case_mgr_t *uc_mgr,
                      const char *identifier,
@@ -370,8 +383,25 @@ int snd_use_case_set(snd_use_case_mgr_t *uc_mgr,
  * \param uc_mgr Returned use case manager pointer
  * \param card_name Sound card name.
  * \return zero if success, otherwise a negative error code
+ *
+ * By default only first card is used when the driver card
+ * name or long name is passed in the card_name argument.
+ *
+ * The "strict:" prefix in the card_name defines that
+ * there is no driver name / long name matching. The straight
+ * configuration is used.
+ *
+ * The "hw:" prefix in the card_name will load the configuration
+ * for the ALSA card specified by the card index (value) or
+ * the card string identificator.
+ *
+ * The sound card might be also composed from several physical
+ * sound cards (for the default and strict card_name).
+ * The application cannot expect that the device names will refer
+ * only one ALSA sound card in this case.
  */
-int snd_use_case_mgr_open(snd_use_case_mgr_t **uc_mgr, const char *card_name);
+int snd_use_case_mgr_open(snd_use_case_mgr_t **uc_mgr,
+                          const char *card_name);
 
 
 /**
@@ -421,6 +451,28 @@ static __inline__ int snd_use_case_verb_list(snd_use_case_mgr_t *uc_mgr,
 	return snd_use_case_get_list(uc_mgr, "_verbs", list);
 }
 
+/**
+ * \brief Parse control element identifier
+ * \param elem_id Element identifier
+ * \param ucm_id Use case identifier
+ * \param value String value to be parsed
+ * \return Zero if success, otherwise a negative error code
+ */
+int snd_use_case_parse_ctl_elem_id(snd_ctl_elem_id_t *dst,
+				   const char *ucm_id,
+				   const char *value);
+
+/**
+ * \brief Parse mixer element identifier
+ * \param dst Simple mixer element identifier
+ * \param ucm_id Use case identifier
+ * \param value String value to be parsed
+ * \return Zero if success, otherwise a negative error code
+ */
+int snd_use_case_parse_selem_id(snd_mixer_selem_id_t *dst,
+				const char *ucm_id,
+				const char *value);
+
 /**
  *  \}
  */
